@@ -13,9 +13,15 @@
 #include "cache.h"
 #include <pwd.h>
 
-static char pathname[PATH_MAX];
 static char bad_path[] = "/bad-path/";
 
+static char *get_pathname(void)
+{
+	static char pathname_array[4][PATH_MAX];
+	static int index;
+	return pathname_array[3 & ++index];
+}
+
 static char *cleanup_path(char *path)
 {
 	/* Clean it up */
@@ -31,6 +37,7 @@ char *mkpath(const char *fmt, ...)
 {
 	va_list args;
 	unsigned len;
+	char *pathname = get_pathname();
 
 	va_start(args, fmt);
 	len = vsnprintf(pathname, PATH_MAX, fmt, args);
@@ -43,6 +50,7 @@ char *mkpath(const char *fmt, ...)
 char *git_path(const char *fmt, ...)
 {
 	const char *git_dir = get_git_dir();
+	char *pathname = get_pathname();
 	va_list args;
 	unsigned len;
 
@@ -77,20 +85,12 @@ int git_mkstemp(char *path, size_t len, const char *template)
 		pch += n;
 	}
 
-	safe_strncpy(pch, template, len);
+	strlcpy(pch, template, len);
 
 	return mkstemp(path);
 }
 
 
-char *safe_strncpy(char *dest, const char *src, size_t n)
-{
-	strncpy(dest, src, n);
-	dest[n - 1] = '\0';
-
-	return dest;
-}
-
 int validate_symref(const char *path)
 {
 	struct stat st;
@@ -250,3 +250,36 @@ char *enter_repo(char *path, int strict)
 
 	return NULL;
 }
+
+int adjust_shared_perm(const char *path)
+{
+	struct stat st;
+	int mode;
+
+	if (!shared_repository)
+		return 0;
+	if (lstat(path, &st) < 0)
+		return -1;
+	mode = st.st_mode;
+	if (mode & S_IRUSR)
+		mode |= (shared_repository == PERM_GROUP
+			 ? S_IRGRP
+			 : (shared_repository == PERM_EVERYBODY
+			    ? (S_IRGRP|S_IROTH)
+			    : 0));
+
+	if (mode & S_IWUSR)
+		mode |= S_IWGRP;
+
+	if (mode & S_IXUSR)
+		mode |= (shared_repository == PERM_GROUP
+			 ? S_IXGRP
+			 : (shared_repository == PERM_EVERYBODY
+			    ? (S_IXGRP|S_IXOTH)
+			    : 0));
+	if (S_ISDIR(mode))
+		mode |= S_ISGID;
+	if ((mode & st.st_mode) != mode && chmod(path, mode) < 0)
+		return -2;
+	return 0;
+}
