@@ -1,8 +1,9 @@
 #include "cache.h"
 #include "run-command.h"
 #include <sys/wait.h>
+#include "exec_cmd.h"
 
-int run_command_v_opt(int argc, char **argv, int flags)
+int run_command_v_opt(int argc, const char **argv, int flags)
 {
 	pid_t pid = fork();
 
@@ -13,22 +14,26 @@ int run_command_v_opt(int argc, char **argv, int flags)
 			int fd = open("/dev/null", O_RDWR);
 			dup2(fd, 0);
 			dup2(fd, 1);
-			close(fd);			
+			close(fd);
+		}
+		if (flags & RUN_GIT_CMD) {
+			execv_git_cmd(argv);
+		} else {
+			execvp(argv[0], (char *const*) argv);
 		}
-		execvp(argv[0], (char *const*) argv);
 		die("exec %s failed.", argv[0]);
 	}
 	for (;;) {
 		int status, code;
-		int retval = waitpid(pid, &status, 0);
+		pid_t waiting = waitpid(pid, &status, 0);
 
-		if (retval < 0) {
+		if (waiting < 0) {
 			if (errno == EINTR)
 				continue;
-			error("waitpid failed (%s)", strerror(retval));
+			error("waitpid failed (%s)", strerror(errno));
 			return -ERR_RUN_COMMAND_WAITPID;
 		}
-		if (retval != pid)
+		if (waiting != pid)
 			return -ERR_RUN_COMMAND_WAITPID_WRONG_PID;
 		if (WIFSIGNALED(status))
 			return -ERR_RUN_COMMAND_WAITPID_SIGNAL;
@@ -42,7 +47,7 @@ int run_command_v_opt(int argc, char **argv, int flags)
 	}
 }
 
-int run_command_v(int argc, char **argv)
+int run_command_v(int argc, const char **argv)
 {
 	return run_command_v_opt(argc, argv, 0);
 }
@@ -50,7 +55,7 @@ int run_command_v(int argc, char **argv)
 int run_command(const char *cmd, ...)
 {
 	int argc;
-	char *argv[MAX_RUN_COMMAND_ARGS];
+	const char *argv[MAX_RUN_COMMAND_ARGS];
 	const char *arg;
 	va_list param;
 
